import { Command } from "discord-akairo";
import { TextChannel } from "discord.js";
import { Message } from "discord.js";
import { cancelString } from "../../config/Constants";
import { Appeal } from "../../entity/Appeal";
import { errorEmbed, infoEmbed, successEmbed } from "../Util";

class AcceptCommand extends Command {
  constructor() {
    super("accept", {
      category: "Review",
      aliases: ["Accept"],
      description: "<appeal> [decision]",
      clientPermissions: ["CREATE_INSTANT_INVITE", "BAN_MEMBERS"],
      channel: "guild",
      args: [
        {
          id: "appeal",
          type: "appeal",
          prompt: {
            start: () => infoEmbed("The appeal ID:").setFooter(cancelString),
          },
        },
        {
          id: "reason",
          type: "string",
          prompt: {
            start: () =>
              infoEmbed("(Optional) Decision note for appeal:")
                .setDescription("Enter `none` to provide no reason")
                .setFooter(cancelString),
          },
        },
      ],
    });
  }

  async banned(
    message: Message,
    args: { appeal: Appeal; reason: string },
    origMsg: Message,
  ): Promise<boolean> {
    let { appeal, reason } = args;
    if (reason === "none") reason = null;

    const banned = await message.guild
      .fetchBan(appeal.userID)
      .then(() => true)
      .catch(() => false);

    if (!banned) {
      message.channel.send(
        infoEmbed("The user is not banned. Appeal status updated.")
          .addField("Original Message", `[link](${appeal.link()}).`)
          .setFooter(appeal.id)
      );

      appeal.accept(message.author.id, reason);
      const embed = appeal.embed(true).setColor("#00AE86");
      origMsg.edit(embed);
    }

    return banned;
  }

  async exec(message: Message, args: { appeal: Appeal; reason: string }) {
    let { appeal, reason } = args;
    if (reason === "none") reason = null;

    const channel = await message.client.channels
      .fetch(appeal.guild.channel)
      .catch(() => null);
    if (!channel || !channel?.isText()) {
      return errorEmbed("Could not update. Guild configuration invalid.");
    }

    const origMsg = await (channel as TextChannel).messages
      .fetch(appeal.message)
      .catch(() => null);
    if (!origMsg) {
      return errorEmbed("Could not update original message");
    }

    if (["accepted", "rejected"].includes(appeal.status)) {
      return message.channel.send(
        errorEmbed("Could not accept")
          .addField("Error", "Cannot accept appeal with decision already made.")
          .addField("Status", appeal.status)
          .setFooter(appeal.id)
      );
    }

    if (!(await this.banned(message, args, origMsg))) return;

    let err: Error = await message.guild.members
      .unban(appeal.userID)
      .then(() => null)
      .catch((e) => e);

    if (err) {
      message.channel.send(
        errorEmbed("An error occurred")
          .setDescription("Could not unban user.")
          .addField("Original Message", `[link](${appeal.link()}).`)
          .addField("Status", "The appeal NOT updated.")
          .addField("Error", err.message)
          .setFooter(appeal.id)
      );
      return;
    }

    appeal.accept(message.author.id, reason);
    origMsg.edit(appeal.embed(true).setColor("#00AE86"));

    message.channel.send(
      successEmbed("The appeal has been accepted.")
        .setDescription(`${appeal.userTag} is unbanned`)
        .addField("Original Message", `[link](${appeal.link()})`)
        .setFooter(appeal.id)
    );

    appeal.save();
  }
}

export default AcceptCommand;
