import { Command } from "discord-akairo";
import { Message } from "discord.js";
import { cancelString } from "../../config/Constants";
import { Guild } from "../../entity/Guild";
import { infoEmbed, successEmbed } from "../Util";

class SetDefaultExpiry extends Command {
  constructor() {
    super("setDefaultExpiry", {
      category: "Server Settings",
      aliases: ["SetDefaultExpiry"],
      description: "<time>",
      clientPermissions: ["CREATE_INSTANT_INVITE", "BAN_MEMBERS"],
      channel: "guild",
      args: [
        {
          id: "newExpiry",
          type: "timestring",
          prompt: {
            start: () =>
              infoEmbed("The timestring:")
                .setDescription(
                  "The minimum default time before users may appeal again after a rejection.\n" +
                    "For no expiry, specify `never`"
                )
                .addField(
                  "Valid Durations",
                  "```1d - days\n1w - weeks\n1m - months\n1y - years```"
                )
                .addField(
                  "Examples",
                  "```\n1m    - 1 month\n1d    - 1 day\n1m 3d - 1 month 3 days\n1y1m  - 1 year 1 month```"
                )
                .setFooter(cancelString),
          },
        },
      ],
    });
  }

  async exec(message: Message, args: { newExpiry: string }) {
    const { newExpiry } = args;
    const author = message.member;

    const guild = await Guild.findOneOrFail({
      where: { guild: author.guild.id },
    });
    guild.defaultExpiry = newExpiry;
    guild.save();

    message.reply(
      successEmbed("Guild Settings Updated")
        .addField("Name", guild.name, true)
        .addField("Post Channel", `<#${guild.channel}>`, true)
        .addField("\u200b", "\u200b", true)
        .addField("Staff Role", `<@&${guild.staffRole}>`, true)
        .addField("Restricted Role", `<@&${guild.jailedRole}>`, true)
        .addField("\u200b", "\u200b", true)
        .addField("Minimum time before re-appeal", guild.defaultExpiry)
    );
  }
}

export default SetDefaultExpiry;
